"""Ошибки"""


class IncorrectDataRecivedError(Exception):
    """
    Исключение  - некорректные данные получены от сокета.
    """
    def __str__(self):
        return 'Принято некорректное сообщение от удалённого компьютера.'


class NonDictInputError(Exception):
    """
    Исключение - аргумент функции не словарь.
    """
    def __str__(self):
        return 'Аргумент функции должен быть словарём.'


class ReqFieldMissingError(Exception):
    """
    Ошибка - отсутствует обязательное поле в принятом словаре.
    """
    def __init__(self, missing_field):
        self.missing_field = missing_field

    def __str__(self):
        return f'В принятом словаре почему-то отсутствует обязательное поле {self.missing_field}.'


class JIMError(Exception):
    """
    Исключение - ошибка протокола JIM.
    """
    def __init__(self, text=''):
        self.text = text

    def __str__(self):
        if len(self.text) > 0:
            return f'Ошибка протокола JIM: {self.text}'
        else:
            return f'Ошибка протокола JIM'


class ServerError(Exception):
    """
    Исключение - ошибка сервера.
    """
    def __init__(self, text):
        self.text = text

    def __str__(self):
        return self.text



"""Декораторы"""
import sys
import logging
sys.path.append('../')
import logs.config_server_log
import logs.config_client_log
import traceback
import inspect

# метод определения модуля, источника запуска.
# Метод find () возвращает индекс первого вхождения искомой подстроки,
# если он найден в данной строке.
# Если его не найдено, - возвращает -1.
if sys.argv[0].find('client') > -1:
    # если не клиент то сервер!
    LOGGER = logging.getLogger('client')
else:
    # ну, раз не сервер, то клиент
    LOGGER = logging.getLogger('server')


# Реализация в виде класса
class Log:
    """Класс-декоратор"""
    def __call__(self, func):
        def log_saver(*args, **kwargs):
            """Завернуто в декоратор."""

            LOGGER.debug(f'Была вызвана функция {func.__name__} c параметрами {args}, {kwargs}. '
                         f'Используемый модуль: {func.__module__} и '
                         f'функция {traceback.format_stack()[0].strip().split()[-1]}. '
                         f'Также была вызвана функция {inspect.stack()[1][3]}.'
                         )

            note = func(*args, **kwargs)
            return note
        return log_saver



import socket
import select
import hmac
import binascii
import os
from sys import path
path.append('../')
from common.decorator import Log, LOGGER
from common.jimbase import JIMBase
from common.json_messenger import JSONMessenger
from common.descriptors import CheckPort, CheckHost
from common.metaclasses import ServerInspector


# class JIMServer(JIMBase, metaclass=ServerInspector):
class JIMServer(JIMBase):
    """Основной класс сервера."""
    transport = None
    clients = []
    messages = []
    # Словарь, содержащий имена пользователей и соответствующие им сокеты.
    messengers = dict()
    listen_address = CheckHost()
    listen_port = CheckPort()
    database = None
    db_session = None
    on_connections_change = []

    # @Log()
    def start(self):
        """Метод запуска сервера"""
        LOGGER.info(f'Запущен сервер, порт для подключений: {self.listen_port}, '
                    f'адрес с которого принимаются подключения: {self.listen_address}. '
                    f'Если адрес не указан, принимаются соединения с любых адресов.')
        # Готовим сокет
        self.transport = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.transport.bind((self.listen_address, self.listen_port))
        self.transport.settimeout(0.5)

        # Слушаем порт
        self.transport.listen(self.MAX_CONNECTIONS)

        # Открываем соединение с базой данных
        self.db_session = self.database.create_session()

    def process(self):
        """Основной цикл программы сервера"""
        # Ждём подключения, если таймаут вышел, ловим исключение.
        try:
            client, client_address = self.transport.accept()
        except OSError:
            pass
        else:
            LOGGER.info(f'Установлено соединение с ПК {client_address}')
            self.clients.append(client)

        recv_data_lst = []
        send_data_lst = []
        err_lst = []

        # Проверяем на наличие ждущих клиентов
        try:
            if self.clients:
                recv_data_lst, send_data_lst, err_lst = select.select(self.clients, self.clients, [], 5)
        except OSError:
            pass

        # принимаем сообщения и если там есть сообщения,
        # кладём в словарь, если ошибка, исключаем клиента.
        if recv_data_lst:
            LOGGER.info('Есть сообщения от клиентов')
            for client_with_message in recv_data_lst:
                try:
                    messenger = JSONMessenger(client_with_message)
                    message = messenger.get_message()
                    self.process_client_message(messenger, message)
                except ConnectionResetError:
                    LOGGER.info(f'Клиент {client_with_message.getpeername()} '
                                f'отключился от сервера.')
                    self.remove_client(client_with_message)
                except Exception as e:
                    LOGGER.info(f'Ошибка при получении сообщения: {e}')
                    self.remove_client(client_with_message)

        # Если есть сообщения, обрабатываем каждое.
        for i in self.messages:
            try:
                self.process_message(i, send_data_lst)
            except Exception:
                LOGGER.info(f'Связь с клиентом с именем {i[self.DESTINATION]} была потеряна')
                self.remove_client(self.messengers[i[self.DESTINATION]].sock)
        self.messages.clear()

    def remove_client(self, sock):
        """Удаление клиента."""
        self.clients.remove(sock)
        for name, messenger in self.messengers.items():
            if messenger.sock == sock:
                del self.messengers[name]
                self.db_session.user_logout(name)
                self.fire(self.on_connections_change)
                break

    @Log()
    def process_client_message(self, messenger, message):
        """
        Обработчик сообщений от клиентов, принимает словарь -
        сообщение от клинта, проверяет корректность,
        возвращает словарь-ответ для клиента.

        :param messenger: экземпляр класса JSONMessenger
        :param message: словарь, полученный от клиента
        :return: возвращает словарь с ответом сервера
        """
        LOGGER.info(f'Разбор сообщения от клиента : {message}')

        if self.ACTION not in message:
            # Иначе отдаём Bad request
            messenger.send_message(self.BAD_REQUEST)
            return

        if message[self.ACTION] == self.PRESENCE \
                and self.TIME in message and self.USER in message and self.ACCOUNT_NAME in message[self.USER]:
            # {'action': 'presence', 'time': 1573760672.167031, 'user': {'account_name': 'Guest'}}
            self.authorize_user(messenger, message)

        elif message[self.ACTION] == self.MESSAGE \
                and self.TIME in message and self.MESSAGE_TEXT in message and self.DESTINATION in message \
                and self.SENDER in message:
            self.messages.append(message)
            self.db_session.process_message(self.SENDER, self.DESTINATION)
            LOGGER.info(f'Сообщение от клиента добавлено в очередь')

        # Если клиент выходит
        elif self.ACTION in message and message[self.ACTION] == self.EXIT and self.ACCOUNT_NAME in message:
            client_name = message[self.ACCOUNT_NAME]
            if self.messengers[client_name] == messenger:
                self.db_session.user_logout(message[self.ACCOUNT_NAME])
                LOGGER.info(f'Клиент {message[self.ACCOUNT_NAME]} корректно отключился от сервера.')
                self.clients.remove(self.messengers[message[self.ACCOUNT_NAME]])
                self.messengers[message[self.ACCOUNT_NAME]].close()
                del self.messengers[message[self.ACCOUNT_NAME]]
                self.fire(self.on_connections_change)

        # Если это запрос контакт-листа
        elif self.ACTION in message and message[self.ACTION] == self.GET_CONTACTS and self.USER in message and \
                self.messengers[message[self.USER]].sock == messenger.sock:
            response = self.RESPONSE_202
            response[self.LIST_INFO] = self.db_session.get_contacts(message[self.USER])
            messenger.send_message(response)

        # Если это добавление контакта
        elif self.ACTION in message and message[self.ACTION] == self.ADD_CONTACT and self.ACCOUNT_NAME in message \
                and self.USER in message and self.messengers[message[self.USER]].sock == messenger.sock:
            self.db_session.add_contact(message[self.USER], message[self.ACCOUNT_NAME])
            response = {self.RESPONSE: 200}
            messenger.send_message(response)

        # Если это удаление контакта
        elif self.ACTION in message and message[self.ACTION] == self.REMOVE_CONTACT and self.ACCOUNT_NAME in message \
                and self.USER in message and self.messengers[message[self.USER]].sock == messenger.sock:
            self.db_session.remove_contact(message[self.USER], message[self.ACCOUNT_NAME])
            response = {self.RESPONSE: 200}
            messenger.send_message(response)

        # Если это запрос известных пользователей
        elif self.ACTION in message and message[self.ACTION] == self.USERS_REQUEST and self.ACCOUNT_NAME in message \
                and self.messengers[message[self.ACCOUNT_NAME]].sock == messenger.sock:
            response = self.RESPONSE_202
            response[self.LIST_INFO] = [user[0] for user in self.db_session.users_list()]
            messenger.send_message(response)

        # Если это запрос публичного ключа пользователя
        elif self.ACTION in message and message[self.ACTION] == self.PUBLIC_KEY_REQUEST and self.ACCOUNT_NAME in message:
            response = self.RESPONSE_511
            response[self.DATA] = self.db_session.get_pubkey(message[self.ACCOUNT_NAME])
            # может быть, что ключа ещё нет (пользователь никогда не логинился,
            # тогда шлём 400)
            if response[self.DATA]:
                try:
                    messenger.send_message(response)
                except OSError:
                    self.remove_client(messenger.sock)
            else:
                response = self.RESPONSE_400
                response[self.ERROR] = 'Нет публичного ключа для данного пользователя'
                try:
                    messenger.send_message(response)
                except OSError:
                    self.remove_client(messenger.sock)

        # Иначе отдаём Bad request
        else:
            messenger.send_message(self.BAD_REQUEST)

    def authorize_user(self, messenger, message):
        """Авторизация пользователя"""
        client_name = message[self.USER][self.ACCOUNT_NAME]
        if client_name in self.messengers.keys():
            response = self.RESPONSE_400
            response[self.ERROR] = 'Имя пользователя уже занято.'
            messenger.send_message(response)
            self.clients.remove(messenger.sock)
            messenger.sock.close()
            return

            # Проверяем что пользователь зарегистрирован на сервере.
        if not self.db_session.check_user(client_name):
            response = self.RESPONSE_400
            response[self.ERROR] = 'Пользователь не зарегистрирован.'
            try:
                LOGGER.debug(f'Unknown username, sending {response}')
                messenger.send_message(response)
            except OSError:
                pass
            self.clients.remove(messenger.sock)
            messenger.sock.close()
            return

        LOGGER.debug('Correct username, starting passwd check.')
        # Иначе отвечаем 511 и проводим процедуру авторизации
        # Словарь - заготовка
        message_auth = self.RESPONSE_511
        # Набор байтов в hex представлении
        random_str = binascii.hexlify(os.urandom(64))
        # В словарь байты нельзя, декодируем (json.dumps -> TypeError)
        message_auth[self.DATA] = random_str.decode('ascii')
        # Создаём хэш пароля и связки с рандомной строкой, сохраняем
        # серверную версию ключа
        hash = hmac.new(self.db_session.get_hash(client_name), random_str, 'MD5')
        digest = hash.digest()
        LOGGER.debug(f'Auth message = {message_auth}')
        try:
            # Обмен с клиентом
            messenger.send_message(message_auth)
            ans = messenger.get_message()
        except OSError as err:
            LOGGER.debug('Error in auth, data:', exc_info=err)
            self.clients.remove(messenger.sock)
            messenger.sock.close()
            return

        client_digest = binascii.a2b_base64(ans[self.DATA])
        # Если ответ клиента корректный, то сохраняем его в список
        # пользователей.
        if not (self.RESPONSE in ans and ans[self.RESPONSE] == 511 and hmac.compare_digest(
                digest, client_digest)):
            response = self.RESPONSE_400
            response[self.ERROR] = 'Неверный пароль.'
            try:
                messenger.send_message(response)
            except OSError:
                pass
            self.clients.remove(messenger.sock)
            messenger.sock.close()

        self.messengers[client_name] = messenger
        client_ip, client_port = messenger.sock.getpeername()
        try:
            messenger.send_message(self.RESPONSE_200)
        except OSError:
            self.remove_client(client_name)
        # добавляем пользователя в список активных и если у него изменился открытый ключ
        # сохраняем новый
        client_pubkey = message[self.USER][self.PUBLIC_KEY]
        self.db_session.user_login(client_name, client_ip, client_port, client_pubkey)
        self.fire(self.on_connections_change)

    @Log()
    def process_message(self, message, listen_socks):
        """
        Метод адресной отправки сообщения определённому клиенту. Принимает словарь сообщение,
        список зарегистрированых пользователей и слушающие сокеты. Ничего не возвращает.
        :param message: словарь, полученный от клиента
        :param listen_socks: слушающий сокет
        :return: нет return, вызывается либо функция отправки сообщения либо логгируется ошибка
        """
        if message[self.DESTINATION] in self.messengers:
            if self.messengers[message[self.DESTINATION]].sock in listen_socks:
                messenger = self.messengers[message[self.DESTINATION]]
                messenger.send_message(message)
                LOGGER.info(f'Отправлено сообщение пользователю {message[self.DESTINATION]} '
                            f'от пользователя {message[self.SENDER]}.')
            else:
                raise ConnectionError
        else:
            LOGGER.error(
                f'Пользователь {message[self.DESTINATION]} не зарегистрирован на сервере, '
                f'отправка сообщения невозможна.')

    def service_update_lists(self):
        """Метод реализующий отправки сервисного сообщения 205 клиентам."""
        for messenger in self.messengers:
            try:
                messenger.send_message(self.RESPONSE_205)
            except OSError:
                self.remove_client(messenger.sock)

    @staticmethod
    def fire(subscription_list):
        for callback in subscription_list:
            callback()


def server_func(my_server):
    while True:
        my_server.process()


import time
from threading import Thread


def show_help():
    print("""
            Поддерживаемые комманды:
            help - это меню
            users - общий список пользователей
            conn - пользователи онлайн
            lh - история входов пользователя
            exit - завершение работы сервера
    """)


def interface_func(database):
    session = database.create_session()
    show_help()
    while True:
        command = input('Введите комманду: ')
        if command == 'help':
            show_help()
        elif command == 'exit':
            break
        elif command == 'users':
            for user in sorted(session.users_list()):
                print({user[0]})
        elif command == 'conn':
            for user in sorted(session.active_users_list()):
                print(f'Пользователь {user[0]}, подключен: {user[1]}:{user[2]}, время установки соединения: {user[3]}')
        elif command == 'lh':
            name = input('Введите имя конкретного пользователя. Для вывода всей истории, просто нажмите Enter: ')
            for user in session.login_history(name):
                print(f'Пользователь: {user[0]} время входа: {user[1]}. Вход с: {user[2]}:{user[3]}')
        else:
            print('Команда не распознана.')


def run_server_cli(server_thread, database):
    # интерфейс - другим
    interface_thread = Thread(target=interface_func, args=(database,))
    interface_thread.daemon = True
    interface_thread.start()

    while True:
        time.sleep(1)
        if server_thread.is_alive() and interface_thread.is_alive():
            continue
        break



import sys
sys.path.append('../')
from common.decorator import LOGGER


class AbstractServerConfig:
    """
    Инициализация переменных конфигурационного файла.
    """
    address = None
    port = None
    db_path = None
    db_file = None


class DefaultServerConfig(AbstractServerConfig):
    """
    Значения конфигурации по умолчанию.
    """
    def __init__(self):
        self.address = '127.0.0.1'
        self.port = 7777
        self.db_path = ''
        self.db_file = '../server_database.db3'


class ServerConfig(AbstractServerConfig):
    """
    Класс для конфигурации сервера.
    """
    default = DefaultServerConfig()

    def read_default(self):
        """Чтение значений по умолчанию, если никаких не введено."""
        self.address = self.default.address
        self.port = self.default.port
        self.db_path = self.default.db_path
        self.db_file = self.default.db_file

    def read_from_ini(self, ini):
        """Чтение конфигурации сервера с файла server.ini."""
        if ini.has_section('SETTINGS'):
            self.address = ini.get('SETTINGS', 'Default_address', fallback=self.default.address)
            self.port = ini.getint('SETTINGS', 'Default_port', fallback=self.default.port)
            self.db_path = ini.get('SETTINGS', 'Database_path', fallback=self.default.db_path)
            self.db_file = ini.get('SETTINGS', 'Database_file', fallback=self.default.db_file)

    def write_to_ini(self, ini):
        """Запись конфигурации сервера в файла server.ini."""
        if not ini.has_section('SETTINGS'):
            ini.add_section('SETTINGS')
        ini['SETTINGS']['Default_port'] = self.address
        ini['SETTINGS']['Default_port'] = str(self.port)
        ini['SETTINGS']['Database_path'] = self.db_path
        ini['SETTINGS']['Database_file'] = self.db_file

    def read_from_cmd(self):
        """
        Чтение с командной строки, например, server.py -p 8079 -a 192.168.1.2.
        """
        try:
            if '-p' in sys.argv:
                self.port = sys.argv[sys.argv.index('-p') + 1]
        except IndexError:
            LOGGER.critical('После параметра -\'p\' необходимо указать номер порта.')
            raise SystemError

        try:
            if '-a' in sys.argv:
                self.address = sys.argv[sys.argv.index('-a') + 1]
        except IndexError:
            LOGGER.critical('После параметра \'a\'- необходимо указать адрес, который будет слушать сервер.')
            raise SystemError

    def copy_to(self, config):
        config.address = self.address
        config.port = self.port
        config.db_path = self.db_path
        config.db_file = self.db_file

    def validate(self):
        """Проверка, что конфигурация корректна."""
        try:
            self.port = int(self.port)
        except ValueError:
            raise ValueError('Порт должен быть числом')
        if not 1023 < self.port < 65536:
            raise ValueError('Порт должен быть числом от 1024 до 65535')



import sys
import configparser
from threading import Lock
from PyQt5.QtWidgets import QMainWindow, QAction, qApp, QApplication, QLabel, QTableView, QDialog, QPushButton, \
    QLineEdit, QFileDialog, QMessageBox
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtCore import Qt, QTimer
sys.path.append('../')
from common.decorator import LOGGER
from server_part.server_config import ServerConfig
from server_part.add_user import RegisterUser
from server_part.remove_user import DelUserDialog

# Флаг что был подключён новый пользователь, нужен чтобы не мучать БД
# постоянными запросами на обновление
new_connection = False
conflag_lock = Lock()


def gui_create_model(db_session):
    """
    GUI - Создание таблицы QModel, для отображения в окне программы.
    :param db_session: сессия БД
    :return: список
    """
    list_users = db_session.active_users_list()
    list = QStandardItemModel()
    list.setHorizontalHeaderLabels(['Имя Клиента', 'IP Адрес', 'Порт', 'Время подключения'])
    for row in list_users:
        user, ip, port, time = row
        user = QStandardItem(user)
        user.setEditable(False)
        ip = QStandardItem(ip)
        ip.setEditable(False)
        port = QStandardItem(str(port))
        port.setEditable(False)
        # Уберём милисекунды из строки времени, т.к. такая точность не требуется.
        time = QStandardItem(str(time.replace(microsecond=0)))
        time.setEditable(False)
        list.appendRow([user, ip, port, time])
    return list


def create_stat_model(db_session):
    """
    GUI - Функция реализующая заполнение таблицы историей сообщений.
    :param db_session: сессия БД
    :return: список
    """
    # Список записей из базы
    hist_list = db_session.message_history()

    # Объект модели данных:
    list = QStandardItemModel()
    list.setHorizontalHeaderLabels(
        ['Имя Клиента', 'Последний раз входил', 'Сообщений отправлено', 'Сообщений получено'])
    for row in hist_list:
        user, sent, recvd = row
        user = QStandardItem(user)
        user.setEditable(False)
        sent = QStandardItem(str(sent))
        sent.setEditable(False)
        recvd = QStandardItem(str(recvd))
        recvd.setEditable(False)
        list.appendRow([user, sent, recvd])
    return list


class MainWindow(QMainWindow):
    """
    Класс основного окна.
    """
    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):
        # Кнопка выхода
        exitAction = QAction('Выход', self)
        exitAction.setShortcut('Ctrl+Q')
        exitAction.triggered.connect(qApp.quit)

        # Кнопка обновить список клиентов
        self.refresh_button = QAction('Обновить список', self)

        # Кнопка настроек сервера
        self.config_btn = QAction('Настройки сервера', self)

        # Кнопка регистрации пользователя
        self.register_btn = QAction('Регистрация пользователя', self)

        # Кнопка удаления пользователя
        self.remove_btn = QAction('Удаление пользователя', self)

        # Кнопка вывести историю сообщений
        self.show_history_button = QAction('История клиентов', self)

        # Статусбар
        # dock widget
        self.statusBar()

        # Тулбар
        self.toolbar = self.addToolBar('MainBar')
        self.toolbar.addAction(exitAction)
        self.toolbar.addAction(self.refresh_button)
        self.toolbar.addAction(self.show_history_button)
        self.toolbar.addAction(self.config_btn)
        self.toolbar.addAction(self.register_btn)
        self.toolbar.addAction(self.remove_btn)

        # Настройки геометрии основного окна
        # Поскольку работать с динамическими размерами мы не умеем, и мало времени на изучение, размер окна фиксирован.
        self.setFixedSize(800, 600)
        self.setWindowTitle('Messaging Server alpha release')

        # Надпись о том, что ниже список подключённых клиентов
        self.label = QLabel('Список подключённых клиентов:', self)
        self.label.setFixedSize(240, 15)
        self.label.move(10, 25)

        # Окно со списком подключённых клиентов.
        self.active_clients_table = QTableView(self)
        self.active_clients_table.move(10, 45)
        self.active_clients_table.setFixedSize(780, 400)

        # Последним параметром отображаем окно.
        self.show()


class HistoryWindow(QDialog):
    """
    Класс окна с историей пользователей.
    """
    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):
        # Настройки окна:
        self.setWindowTitle('Статистика клиентов')
        self.setFixedSize(600, 700)
        self.setAttribute(Qt.WA_DeleteOnClose)

        # Кнапка закрытия окна
        self.close_button = QPushButton('Закрыть', self)
        self.close_button.move(250, 650)
        self.close_button.clicked.connect(self.close)

        # Лист с собственно историей
        self.history_table = QTableView(self)
        self.history_table.move(10, 10)
        self.history_table.setFixedSize(580, 620)

        self.show()


class ConfigWindow(QDialog):
    """
    Класс окна настроек.
    """
    def __init__(self):
        super().__init__()
        self.initUI()

    def save(self):
        return

    def initUI(self):
        # Настройки окна
        self.setFixedSize(365, 260)
        self.setWindowTitle('Настройки сервера')

        # Надпись о файле базы данных:
        self.db_path_label = QLabel('Путь до файла базы данных: ', self)
        self.db_path_label.move(10, 10)
        self.db_path_label.setFixedSize(240, 15)

        # Строка с путём базы
        self.db_path = QLineEdit(self)
        self.db_path.setFixedSize(250, 20)
        self.db_path.move(10, 30)
        self.db_path.setReadOnly(True)

        # Кнопка выбора пути.
        self.db_path_select = QPushButton('Обзор...', self)
        self.db_path_select.move(275, 28)

        # Функция обработчик открытия окна выбора папки
        def open_file_dialog():
            global dialog
            dialog = QFileDialog(self)
            path = dialog.getExistingDirectory()
            path = path.replace('/', '\\')
            self.db_path.insert(path)

        self.db_path_select.clicked.connect(open_file_dialog)

        # Метка с именем поля файла базы данных
        self.db_file_label = QLabel('Имя файла базы данных: ', self)
        self.db_file_label.move(10, 68)
        self.db_file_label.setFixedSize(180, 15)

        # Поле для ввода имени файла
        self.db_file = QLineEdit(self)
        self.db_file.move(200, 66)
        self.db_file.setFixedSize(150 , 20)

        # Метка с номером порта
        self.port_label = QLabel('Номер порта для соединений:', self)
        self.port_label.move(10, 108)
        self.port_label.setFixedSize(180, 15)

        # Поле для ввода номера порта
        self.port = QLineEdit(self)
        self.port.move(200, 108)
        self.port.setFixedSize(150, 20)

        # Метка с адресом для соединений
        self.ip_label = QLabel('С какого IP принимаем соединения:', self)
        self.ip_label.move(10, 148)
        self.ip_label.setFixedSize(180, 15)

        # Метка с напоминанием о пустом поле.
        self.ip_label_note = QLabel(' оставьте это поле пустым, чтобы\n принимать соединения с любых адресов.', self)
        self.ip_label_note.move(10, 168)
        self.ip_label_note.setFixedSize(500, 30)

        # Поле для ввода ip
        self.ip = QLineEdit(self)
        self.ip.move(200, 148)
        self.ip.setFixedSize(150, 20)

        # Кнопка сохранения настроек
        self.save_btn = QPushButton('Сохранить', self)
        self.save_btn.move(190 , 220)
        self.save_btn.clicked.connect(self.save)

        # Кнапка закрытия окна
        self.close_button = QPushButton('Закрыть', self)
        self.close_button.move(275, 220)
        self.close_button.clicked.connect(self.close)

        self.show()


def run_server_gui(server, config):
    """
    Запуск GUI сервера.
    :param server: сервер
    :param config: конфигурация
    """
    # Создаём графическое окуружение для сервера:
    server_app = QApplication(sys.argv)
    main_window = MainWindow()
    db_session = server.database.create_session()

    # Инициализируем параметры в окна
    main_window.statusBar().showMessage('Server Working')
    main_window.active_clients_table.setModel(gui_create_model(db_session))
    main_window.active_clients_table.resizeColumnsToContents()
    main_window.active_clients_table.resizeRowsToContents()

    def list_update():
        """
        Подфункция обновляющая список подключённых, проверяет флаг подключения, и
        если надо обновляет список.
        """
        global new_connection
        if new_connection:
            main_window.active_clients_table.setModel(
                gui_create_model(db_session))
            main_window.active_clients_table.resizeColumnsToContents()
            main_window.active_clients_table.resizeRowsToContents()
            with conflag_lock:
                new_connection = False

    def show_statistics():
        """
        Подфункция создающая окно со статистикой клиентов.
        """
        global stat_window
        stat_window = HistoryWindow()
        stat_window.history_table.setModel(create_stat_model(db_session))
        stat_window.history_table.resizeColumnsToContents()
        stat_window.history_table.resizeRowsToContents()
        stat_window.show()

    def server_config():
        """
        Подфункция создающяя окно с настройками сервера.
        """
        global config_window
        # Создаём окно и заносим в него текущие параметры
        config_window = ConfigWindow()
        config_window.db_path.insert(config.db_path)
        config_window.db_file.insert(config.db_file)
        config_window.port.insert(str(config.port))
        config_window.ip.insert(config.address)
        config_window.save_btn.clicked.connect(save_server_config)
        config_window.show()

    def reg_user():
        """Подфункция создающая окно регистрации пользователя."""
        global reg_window
        reg_window = RegisterUser(server)
        reg_window.show()

    def rem_user():
        """Подфункция создающая окно удаления пользователя."""
        global rem_window
        rem_window = DelUserDialog(server)
        rem_window.show()

    def save_server_config():
        """Подфункция сохранения настроек."""
        global config_window
        message = QMessageBox()
        new_config = ServerConfig()
        new_config.db_path = config_window.db_path.text()
        new_config.db_file = config_window.db_file.text()
        new_config.port = config_window.port.text()
        new_config.address = config_window.ip.text()
        try:
            new_config.validate()
            new_config.copy_to(config)
        except ValueError as e:
            message.warning(config_window, 'Ошибка', str(e))
        except Exception as e:
            message.warning(config_window, 'Ошибка', 'Системная ошибка при загрузке конфигурации')
            LOGGER.critical(str(e))
        try:
            with open('../server.ini', 'w') as file:
                ini = configparser.ConfigParser()
                config.write_to_ini(ini)
                ini.write(file)
            message.information(
                config_window, 'OK', 'Настройки успешно сохранены!')
        except Exception as e:
            message.warning(config_window, 'Ошибка', 'Системная ошибка при записи конфигурации в файл')
            LOGGER.critical(str(e))

    # Таймер, обновляющий список клиентов 1 раз в секунду
    timer = QTimer()
    timer.timeout.connect(list_update)
    timer.start(1000)

    def handle_connections_change():
        global new_connection
        with conflag_lock:
            new_connection = True
    server.on_connections_change.append(handle_connections_change)

    # Связываем кнопки с процедурами
    main_window.refresh_button.triggered.connect(list_update)
    main_window.show_history_button.triggered.connect(show_statistics)
    main_window.config_btn.triggered.connect(server_config)
    main_window.register_btn.triggered.connect(reg_user)
    main_window.remove_btn.triggered.connect(rem_user)

    # Запускаем GUI
    server_app.exec_()



"""
Класс с тестами
"""
import sys
import os
import unittest
sys.path.append(os.path.join(os.getcwd(), '..'))
from common.jimbase import JIMBase
from client import JIMClient


class TestMyFunction(unittest.TestCase):
    def test_def_presence(self):
        """Тест коректного запроса с корректным словарем"""
        test = JIMClient.create_presence()
        test[JIMBase.TIME] = 1.3  # время необходимо приравнять принудительно иначе будет ошибка
        self.assertEqual(test, {JIMBase.ACTION: JIMBase.PRESENCE,
                                JIMBase.TIME: 1.3,
                                JIMBase.USER: {JIMBase.ACCOUNT_NAME: 'Guest'}})

    def test_success_ans(self):
        """Тест корректтного разбора ответа Hello!"""
        self.assertEqual(JIMClient.process_ans({JIMBase.RESPONSE: 200}), 'Hello!')

    def test_error_ans(self):
        """Тест корректного разбора Error"""
        self.assertEqual(JIMClient.process_ans({JIMBase.RESPONSE: 400,
                                                JIMBase.ERROR: 'Bad Request'}), 'Error: Bad Request')

    def test_no_response(self):
        """Тест исключения без поля RESPONSE"""
        self.assertRaises(ValueError, JIMClient.process_ans, {JIMBase.ERROR: 'Bad Request'})


if __name__ == '__main__':
    unittest.main()



"""Дескрипторы"""
import logging
logger = logging.getLogger('server')


class CheckPort:
    """
    Дескриптор для описания порта.
    """
    def __set__(self, instance, value):
        # instance - <__main__.Server object at 0x000000D582740C50>
        # value - 7777
        try:
            port = int(value)
        except ValueError:
            raise ValueError('Порт должен быть целым числом!')
        if not 1023 < value < 65536:
            raise ValueError(
                f'Попытка запуска сервера с указанием неподходящего порта {value}. Допустимы адреса с 1024 до 65535.')
        # Если порт прошел проверку, добавляем его в список атрибутов экземпляра
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        # owner - <class '__main__.Server'>
        # name - port
        self.name = name


# Дескриптор для описания IP (домена-то у меня нет...):
class CheckHost:
    """
    Дескриптор для описания IP.
    """
    def __set__(self, instance, value):
        start_ip = value.split('.')  # разделяю каждое число в IP, разделитель - точка
        max_oct = 254  # максимальное число в октете
        min_oct = 0  # минимальное число в октете
        oct_in_ip = 4  # кол-во октетов в ip

        # далее перевожу все числа в int
        start_ip[0] = int(start_ip[0])
        start_ip[1] = int(start_ip[1])
        start_ip[2] = int(start_ip[2])
        start_ip[3] = int(start_ip[3])
        # смотрю, что каждое число не больше 254 и не меньше 0
        i = 0
        if len(start_ip) == oct_in_ip:
            while i < oct_in_ip:  # !!!
                if not (start_ip[i] <= max_oct and start_ip[i] >= min_oct):
                    raise ValueError(
                                     f'Попытка запуска сервера с указанием неподходящего {i+1} октета IP {start_ip[i]}.'
                                     ' Допустим диапазон с 0 до 254.')
                i += 1
            # если не ушло в ошибку ValueError, то октеты корректные
            instance.__dict__[self.name] = value
        else:
            raise ValueError('Октетов должно быть 4!')

    def __set_name__(self, owner, name):
        # owner - <class '__main__.Server'>
        # name - port
        self.name = name




from sys import path
from sqlalchemy import create_engine, Table, Column, Integer, String, MetaData, ForeignKey, DateTime, Text
from sqlalchemy.orm import mapper, sessionmaker
from sqlalchemy.sql import default_comparator
from datetime import datetime
path.append('../')
from common.decorator import LOGGER


# Класс - серверная база данных:
class ServerDB:
    """
    Класс для инициализации базы данных сервера.
    """
    class ChatUsers:
        """Подкласс - отображение таблицы всех пользователей:
            Экземпляр этого класса = запись в таблице AllUsers."""
        def __init__(self, username, passwd_hash):
            self.name = username
            self.last_login = datetime.now()
            self.passwd_hash = passwd_hash
            self.pubkey = None
            self.id = None

    class ActiveUsers:
        """Подкласс - отображение таблицы активных пользователей:
           Экземпляр этого класса = запись в таблице ActiveUsers."""
        def __init__(self, user_id, ip_address, port, login_time):
            self.user = user_id
            self.ip_address = ip_address
            self.port = port
            self.login_time = login_time
            self.id = None

    class LoginHistory:
        """Подкласс - отображение таблицы истории входов:
           Экземпляр этого класса = запись в таблице LoginHistory."""
        def __init__(self, name, date, ip, port):
            self.id = None
            self.name = name
            self.date_time = date
            self.ip = ip
            self.port = port

    class UsersContacts:
        """
        Подкласс - отображение таблицы контактов пользователей.
        """
        def __init__(self, user, contact):
            self.id = None
            self.user = user
            self.contact = contact

    class UsersHistory:
        """
        Подкласс отображение таблицы истории действий.
        """
        def __init__(self, user):
            self.id = None
            self.user = user
            self.sent = 0
            self.accepted = 0


    class ServerDBSession:
        def __init__(self, db):
            self.db = db
            self.session = db.session_maker()

        # Функция выполняющяяся при входе пользователя, записывает в базу факт входа
        def user_login(self, username, ip_address, port, key):
            """
            Метод выполняющийся при входе пользователя, записывает в базу факт входа
            Обновляет открытый ключ пользователя при его изменении.
            :param username: имя пользователя
            :param ip_address: адрес, с которого пользователь зашел
            :param port: порт, с которого пользователь зашел
            :param key: ключ авторизации
            :return: вызываются функции
            """
            # print(username, ip_address, port)
            # Запрос в таблицу пользователей на наличие там пользователя с таким именем
            check_users = self.session.query(self.db.ChatUsers).filter_by(name=username)
            # Если имя пользователя уже присутствует в таблице, обновляем время последнего входа
            if check_users.count():
                user = check_users.first()
                user.last_login = datetime.now()
                if user.pubkey != key:
                    user.pubkey = key
            # Если нету, то генерируем исключение
            else:
                raise ValueError('Пользователь не зарегистрирован.')

            # Теперь можно создать запись в таблицу активных пользователей о факте входа.
            # Создаем экземпляр класса self.ActiveUsers, через который передаем данные в таблицу
            new_active_user = self.db.ActiveUsers(user.id, ip_address, port, datetime.now())
            self.session.add(new_active_user)

            # и сохранить в историю входов
            # Создаем экземпляр класса self.LoginHistory, через который передаем данные в таблицу
            history = self.db.LoginHistory(user.id, datetime.now(), ip_address, port)
            self.session.add(history)

            # Сохраняем изменения
            self.session.commit()

        def add_user(self, name, passwd_hash):
            """
            Метод регистрации пользователя.
            Принимает имя и хэш пароля, создаёт запись в таблице статистики.
            :param name: имя пользователя
            :param passwd_hash: хеш пароля
            :return: вызываются функции
            """
            # Создаем экземпляр класса self.AllUsers, через который передаем данные в таблицу
            user = self.db.ChatUsers(name, passwd_hash)
            self.session.add(user)
            self.session.flush()
            user_history = self.db.UsersHistory(user.id)
            self.session.add(user_history)
            # Комит здесь нужен, чтобы присвоился ID
            self.session.commit()

        def remove_user(self, name):
            """
            Метод удаляющий пользователя из базы.
            :param name: имя пользователя
            :return: вызываются функции
            """
            user = self.session.query(self.db.ChatUsers).filter_by(name=name).first()
            self.session.query(self.db.ActiveUsers).filter_by(user=user.id).delete()
            self.session.query(self.db.LoginHistory).filter_by(name=user.id).delete()
            self.session.query(self.db.UsersContacts).filter_by(user=user.id).delete()
            self.session.query(
                self.db.UsersContacts).filter_by(
                contact=user.id).delete()
            self.session.query(self.db.UsersHistory).filter_by(user=user.id).delete()
            self.session.query(self.db.ChatUsers).filter_by(name=name).delete()
            self.session.commit()

        def get_hash(self, name):
            """
            Метод получения хэша пароля пользователя.
            :param name: имя пользователя
            :return: получение хеша пароля
            """
            user = self.session.query(self.db.ChatUsers).filter_by(name=name).first()
            return user.passwd_hash

        def get_pubkey(self, name):
            """
            Метод получения публичного ключа пользователя.
            :param name: имя пользователя
            :return: получение публичного ключа пользователя
            """
            user = self.session.query(self.db.ChatUsers).filter_by(name=name).first()
            return user.pubkey

        def check_user(self, name):
            """
            Метод проверяющий существование пользователя.
            :param name: имя пользователя
            :return: если пользователь существует - True, иначе - False
            """
            if self.session.query(self.db.ChatUsers).filter_by(name=name).count():
                return True
            else:
                return False

        def user_logout(self, username):
            """
            Метод фиксирующий отключение пользователя.
            :param username: имя пользователя
            :return: вызываются функции
            """
            # Запрашиваем пользователя, что покидает нас
            # получаем запись из таблицы AllUsers
            user = self.session.query(self.db.ChatUsers).filter_by(name=username).first()

            # Удаляем его из таблицы активных пользователей.
            # Удаляем запись из таблицы ActiveUsers
            self.session.query(self.db.ActiveUsers).filter_by(user=user.id).delete()

            # Применяем изменения
            self.session.commit()

        def process_message(self, sender_name, recipient_name):
            """
            Метод фиксирует передачу сообщения и делает соответствующие отметки в БД.
            :param sender_name: имя отправителя
            :param recipient_name: имя получателя
            :return: если нет получателя или отправителя, то ошибка
            """
            # Получаем строки отправителя и получателя
            sender = self.session.query(self.db.ChatUsers).filter_by(name=sender_name).first()
            if sender is None:
                LOGGER.error(f'process_message не нашел пользователя "{sender_name}" в таблице ChatUsers')
                return
            recipient = self.session.query(self.db.ChatUsers).filter_by(name=recipient_name).first()
            if recipient is None:
                LOGGER.error(f'process_message не нашел пользователя "{recipient_name}" в таблице ChatUsers')
                return
            # Запрашиваем строки из истории и увеличиваем счётчики
            sender_row = self.session.query(self.db.UsersHistory).filter_by(user=sender.id).first()
            sender_row.sent += 1
            recipient_row = self.session.query(self.db.UsersHistory).filter_by(user=recipient.id).first()
            recipient_row.accepted += 1

            self.session.commit()

        def add_contact(self, user, contact):
            """
            Метод добавляет контакт для пользователя.
            :param user: Пользователь
            :param contact: контакт
            :return: если дубль - получаем ошибку
            """
            # Получаем ID пользователей
            user = self.session.query(self.db.ChatUsers).filter_by(name=user).first()
            contact = self.session.query(self.db.ChatUsers).filter_by(name=contact).first()

            # Проверяем что не дубль и что контакт может существовать (полю пользователь мы доверяем)
            if not contact or self.session.query(self.db.UsersContacts).filter_by(user=user.id,
                                                                                  contact=contact.id).count():
                return

            # Создаём объект и заносим его в базу
            contact_row = self.db.UsersContacts(user.id, contact.id)
            self.session.add(contact_row)
            self.session.commit()

        def remove_contact(self, user, contact):
            """
            Метод удаляет контакт из базы данных.
            :param user: пользователь
            :param contact: контакт
            :return: если контакт не может существовать - ошибка
            """
            # Получаем ID пользователей
            user = self.session.query(self.db.ChatUsers).filter_by(name=user).first()
            contact = self.session.query(self.db.ChatUsers).filter_by(name=contact).first()

            # Проверяем что контакт может существовать (полю пользователь мы доверяем)
            if not contact:
                return

            # Удаляем требуемое
            print(self.session.query(self.db.UsersContacts).filter(
                self.db.UsersContacts.user == user.id,
                self.db.UsersContacts.contact == contact.id
            ).delete())
            self.session.commit()

        def users_list(self):
            """
            Метод возвращает список известных пользователей со временем последнего входа (в виде кортежей).
            :return: список
            """
            return self.session.query(self.db.ChatUsers.name).all()

        def active_users_list(self):
            """
            Метод возвращает список активных пользователей.
            :return: список
            """
            # Запрашиваем соединение таблиц и собираем кортежи имя, адрес, порт, время.
            return self.session.query(
                self.db.ChatUsers.name,
                self.db.ActiveUsers.ip_address,
                self.db.ActiveUsers.port,
                self.db.ActiveUsers.login_time
            ).join(self.db.ChatUsers).all()

        def login_history(self, username=None):
            """
            Метод возвращающает историю входов по пользователю или всем пользователям.
            :param username: пользователь
            :return: история входов
            """
            # Запрашиваем историю входа
            query = self.session.query(self.db.ChatUsers.name,
                                       self.db.LoginHistory.date_time,
                                       self.db.LoginHistory.ip,
                                       self.db.LoginHistory.port
                                       ).join(self.db.ChatUsers).order_by(self.db.LoginHistory.date_time)
            # Если было указано имя пользователя, то фильтруем по нему
            if username:
                query = query.filter(self.db.ChatUsers.name == username)
            return query.all()

        def get_contacts(self, username):
            """
            Метод возвращает список контактов пользователя.
            :param username: пользователь
            :return: список
            """
            # Запрашивааем указанного пользователя
            user = self.session.query(self.db.ChatUsers).filter_by(name=username).one()

            # Запрашиваем его список контактов
            query = self.session.query(self.db.UsersContacts, self.db.ChatUsers.name). \
                filter_by(user=user.id). \
                join(self.db.ChatUsers, self.db.UsersContacts.contact == self.db.ChatUsers.id)

            # выбираем только имена пользователей и возвращаем их.
            return [contact[1] for contact in query.all()]

        def message_history(self):
            """
            Метод возвращает количество переданных и полученных сообщений.
            :return: количество переданных и полученных сообщений
            """
            query = self.session.query(
                self.db.ChatUsers.name,
                self.db.UsersHistory.sent,
                self.db.UsersHistory.accepted
            ).join(self.db.ChatUsers)
            # Возвращаем список кортежей
            return query.all()

    def __init__(self, path):
        # Создаём движок базы данных
        # SERVER_DATABASE - sqlite:///server_database.db3
        # echo=False - отключаем ведение лога (вывод sql-запросов)
        # pool_recycle - По умолчанию соединение с БД через 8 часов простоя обрывается.
        # Чтобы это не случилось нужно добавить опцию pool_recycle = 7200 (переуст-ка соед-я через 2 часа)
        self.database_engine = create_engine(f'sqlite:///{path}', echo=False, pool_recycle=7200,
                                             connect_args={'check_same_thread': False})

        # Создаём объект MetaData
        self.metadata = MetaData()

        # Создаём таблицу пользователей
        chat_users_table = Table('Chat_Users', self.metadata,
                                 Column('id', Integer, primary_key=True),
                                 Column('name', String, unique=True),
                                 Column('passwd_hash', String),
                                 Column('pubkey', Text)
                                 )

        # Создаём таблицу активных пользователей
        active_users_table = Table('Active_Users', self.metadata,
                                   Column('id', Integer, primary_key=True),
                                   Column('user', ForeignKey('Chat_Users.id'), unique=True),
                                   Column('ip_address', String),
                                   Column('port', Integer),
                                   Column('login_time', DateTime)
                                   )

        # Создаём таблицу истории входов
        login_history_table = Table('Login_History', self.metadata,
                                    Column('id', Integer, primary_key=True),
                                    Column('name', ForeignKey('Chat_Users.id')),
                                    Column('date_time', DateTime),
                                    Column('ip', String),
                                    Column('port', String)
                                    )

        # Создаём таблицу контактов пользователей
        users_contacts_table = Table('Users_Contacts', self.metadata,
                                     Column('id', Integer, primary_key=True),
                                     Column('user', ForeignKey('Chat_Users.id')),
                                     Column('contact', ForeignKey('Chat_Users.id'))
                                     )

        # Создаём таблицу истории пользователей
        users_history_table = Table('Users_History', self.metadata,
                                    Column('id', Integer, primary_key=True),
                                    Column('user', ForeignKey('Chat_Users.id')),
                                    Column('sent', Integer),
                                    Column('accepted', Integer)
                                    )

        # Создаём таблицы
        self.metadata.create_all(self.database_engine)

        # Создаём отображения
        # Связываем класс в ORM с таблицей
        mapper(self.ChatUsers, chat_users_table)
        mapper(self.ActiveUsers, active_users_table)
        mapper(self.LoginHistory, login_history_table)
        mapper(self.UsersContacts, users_contacts_table)
        mapper(self.UsersHistory, users_history_table)

        # Создаём сессию
        self.session_maker = sessionmaker(bind=self.database_engine)
        session = self.session_maker()

        # Если в таблице активных пользователей есть записи, то их необходимо удалить
        # Когда устанавливаем соединение, очищаем таблицу активных пользователей
        session.query(self.ActiveUsers).delete()
        session.commit()

    def create_session(self):
        """
        Создаем сессию
        :return: создание сессии к БД
        """
        return self.ServerDBSession(self)


# Отладка
if __name__ == '__main__':
    test_db = ServerDB()
    session = test_db.create_session()
    # выполняем 'подключение' пользователя и выводим список активных пользователей
    session.user_login('client_1', '192.168.1.4', 8888)
    print(session.active_users_list())
    # подключаем другого пользователя и выводим список активных пользователей
    session.user_login('client_2', '192.168.1.5', 7777)
    print(session.active_users_list())
    # выполянем 'отключение' пользователя
    session.user_logout('client_1')
    # выводим список активных пользователей
    print(session.active_users_list())
    # пользователь опять заходит и выходит
    session.user_login('client_1', '192.168.1.4', 8888)
    session.user_logout('client_1')
    # запрашиваем историю входов по пользователю
    print(session.login_history('client_1'))
    # выводим список известных пользователей
    print(session.users_list())
    session.add_contact('test2', 'test1')
    session.add_contact('test1', 'test3')
    session.add_contact('test1', 'test6')
    session.remove_contact('test1', 'test3')
    session.process_message('client_1', 'client_2')
    print(session.message_history())



"""Утилиты"""
import json
import sys
import os
sys.path.append(os.path.join(os.getcwd(), '..'))
from common.errors import NonDictInputError
from common.decorator import Log


class JSONMessenger:
    """
    Класс для приема и отправки сообщений между клиентом и сервером.
    """
    # Максимальная длинна сообщения в байтах
    MAX_PACKAGE_LENGTH = 1024
    # Кодировка проекта
    ENCODING = 'utf-8'

    def __init__(self, sock):
        self.sock = sock

    @Log()
    def get_message(self):
        """
        Метод приёма и декодирования сообщения
        принимает байты, выдаёт словарь, а если принято что-то другое -- отдаёт ошибку значения.
        :return: возвращает словарь, полученный по сокету
        """
        encoded_response = self.sock.recv(self.MAX_PACKAGE_LENGTH)
        if not isinstance(encoded_response, bytes):
            raise ValueError
        json_response = encoded_response.decode(self.ENCODING)
        response = json.loads(json_response)
        if not isinstance(response, dict):
            raise NonDictInputError
        return response

    @Log()
    def send_message(self, message):
        """
        Метод кодирования и отправки сообщения принимает словарь и отправляет его.
        :param message: сообщение в виде словаря
        """

        js_message = json.dumps(message)
        encoded_message = js_message.encode(self.ENCODING)
        self.sock.send(encoded_message)



from setuptools import setup, find_packages

setup(name="my_mess_server",
      version="0.0.1",
      description="mess_server",
      author="Tatiana Grishechkina",
      author_email="tatiangr@amdocs.com",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex']
      )



"""Константы"""
import os
from sys import path
from time import time
path.append(os.path.join(os.getcwd(), '..'))
from common.decorator import Log, LOGGER


class JIMBase:
    """
    Класс констант.
    """
    # Порт по умолчанию для сетевого ваимодействия
    DEFAULT_PORT = 7777
    # IP адрес по умолчанию для подключения клиента
    DEFAULT_IP_ADDRESS = '127.0.0.1'
    # Максимальная очередь подключений
    MAX_CONNECTIONS = 5
    # База данных для хранения данных сервера:
    SERVER_DATABASE = 'sqlite:///server_database.db3'

    # Прококол JIM основные ключи:
    ACTION = 'action'
    TIME = 'time'
    USER = 'user'
    ACCOUNT_NAME = 'account_name'
    SENDER = 'sender'
    DESTINATION = 'to'
    DATA = 'bin'
    PUBLIC_KEY = 'pubkey'

    # Прочие ключи, используемые в протоколе
    PRESENCE = 'presence'
    RESPONSE = 'response'
    ERROR = 'error'
    MESSAGE = 'message'
    MESSAGE_TEXT = 'mess_text'
    EXIT = 'exit'
    GET_CONTACTS = 'get_contacts'
    LIST_INFO = 'data_list'
    REMOVE_CONTACT = 'remove'
    ADD_CONTACT = 'add'
    USERS_REQUEST = 'get_users'
    PUBLIC_KEY_REQUEST = 'pubkey_need'

    BAD_REQUEST = {
        RESPONSE: 400,
        ERROR: 'Bad Request'
    }
    # 200
    RESPONSE_200 = {
        RESPONSE: 200,
    }
    # 202
    RESPONSE_202 = {
        RESPONSE: 202,
        LIST_INFO: None
    }
    # 400
    RESPONSE_400 = {
        RESPONSE: 400,
        ERROR: None
    }

    # 205
    RESPONSE_205 = {
        RESPONSE: 205
    }

    # 511
    RESPONSE_511 = {
        RESPONSE: 511,
        DATA: None
    }

    @classmethod
    @Log()
    def create_message(cls, text, from_acc_name, to_acc_name):
        """Метод запрашивает текст сообщения и возвращает его.
        Так же завершает работу при вводе подобной комманды.
        """
        message_dict = {
            cls.ACTION: cls.MESSAGE,
            cls.SENDER: from_acc_name,
            cls.DESTINATION: to_acc_name,
            cls.TIME: time(),
            cls.MESSAGE_TEXT: text
        }
        LOGGER.debug(f'Сформирован словарь сообщения: {message_dict}')
        return message_dict



"""Unit-тесты доп файла"""

import sys
import os
import unittest
import json
sys.path.append(os.path.join(os.getcwd(), '..'))
from common.jimbase import JIMBase
from common.json_messenger import JSONMessenger


class TestSocket:
    """
    Тестовый класс для тестирования отправки и получения,
    при создании требует словарь, который будет прогонятся
    через тестовую функцию. Тестовая заглушка, эмулирующая работу сокета.
    """
    def __init__(self, test_dict):
        self.test_dict = test_dict
        self.en_message = None
        self.r_message = None

    def send(self, message_to_send):
        """
        Тестовая функция отправки, корретно  кодирует сообщение,
        так-же сохраняет что должно было отправлено в сокет.
        message_to_send - то, что отправляем в сокет
        :param message_to_send:
        :return:
        """
        json_test_message = json.dumps(self.test_dict)
        # кодирует сообщение
        self.en_message = json_test_message.encode(JSONMessenger.ENCODING)
        # сохраняем что должно было отправлено в сокет
        self.r_message = message_to_send

    def recv(self, max_len):
        """
        Получаем данные из сокета
        :param max_len:
        :return:
        """
        json_test_message = json.dumps(self.test_dict)
        return json_test_message.encode(JSONMessenger.ENCODING)


class Tests(unittest.TestCase):
    """Тестовый класс, собственно выполняющий тестирование."""
    test_dict_send = {
        JIMBase.ACTION: JIMBase.PRESENCE,
        JIMBase.TIME: 111111.111111,
        JIMBase.USER: {
            JIMBase.ACCOUNT_NAME: 'test_test'
        }
    }
    test_dict_recv_ok = {JIMBase.RESPONSE: 200}
    test_dict_recv_err = {
        JIMBase.RESPONSE: 400,
        JIMBase.ERROR: 'Bad Request'
    }

    def test_send_message(self):
        """
        Тестируем корректность работы фукции отправки,
        создадим тестовый сокет и проверим корректность отправки словаря
        :return:
        """
        # экземпляр тестового словаря, хранит собственно тестовый словарь
        test_socket = TestSocket(self.test_dict_send)
        # вызов функции send_message, результаты будут сохранены в тестовом сокете test_socket
        JSONMessenger(test_socket).send_message(self.test_dict_send)
        # проверка корретности кодирования словаря.
        # сравниваем результат нашего кодирования и результат от тестируемой функции
        self.assertEqual(test_socket.en_message, test_socket.r_message)
        # дополнительно, проверим исключения, если на входе не словарь.
        with self.assertRaises(Exception):
            JSONMessenger(test_socket).send_message()

    def test_get_message(self):
        """
        Тест функции приёма сообщения с сокета
        :return:
        """
        test_sock_ok = TestSocket(self.test_dict_recv_ok)
        test_sock_err = TestSocket(self.test_dict_recv_err)
        # тест корректной расшифровки корректного словаря
        self.assertEqual(JSONMessenger(test_sock_ok).get_message(), self.test_dict_recv_ok)
        # тест корректной расшифровки ошибочного словаря
        self.assertEqual(JSONMessenger(test_sock_err).get_message(), self.test_dict_recv_err)


if __name__ == '__main__':
    unittest.main()



"""Программа-сервер"""
import os
import configparser
from threading import Thread
from common.decorator import Log, LOGGER
from server_part.server_database import ServerDB
from server_part.server_config import ServerConfig
from server_part.server_gui import run_server_gui
from server_part.main_class import JIMServer, server_func


def main():
    # Загрузка файла конфигурации сервера
    ini = configparser.ConfigParser()
    dir_path = os.getcwd()
    ini.read(f"{dir_path}/{'server.ini'}")

    # Чтение конфигурации сервера
    config = ServerConfig()
    config.read_default()
    config.read_from_ini(ini)
    config.read_from_cmd()

    # Инициализация базы данных
    database = ServerDB(os.path.join(config.db_path, config.db_file))

    # Запуск сервера
    my_server = JIMServer()
    my_server.listen_address = config.address
    my_server.listen_port = config.port
    my_server.database = database
    my_server.start()
    print(f'Сервер запущен на порту {config.address}:{config.port}')

    # Запускаем поток обработки сообщений сервером
    server_thread = Thread(target=server_func, args=(my_server,))
    server_thread.daemon = True
    server_thread.start()

    # Запускаем пользовательский интерфейс
    # run_server_cli(server_thread, database)
    run_server_gui(my_server, config)


if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        LOGGER.critical(str(e))




"""Unit-тесты сервера"""

import sys
import os
import unittest
sys.path.append(os.path.join(os.getcwd(), '..'))
from common.jimbase import JIMBase
from server import JIMServer


class TestServer(unittest.TestCase):
    """В сервере только 1 функция для тестирования"""
    err_dict = {
        JIMBase.RESPONSE: 400,
        JIMBase.ERROR: 'Bad Request'
    }
    ok_dict = {JIMBase.RESPONSE: 200}

    def test_no_action(self):
        """Ошибка если нет ACTION"""
        self.assertEqual(JIMServer.process_client_message(
            {JIMBase.TIME: '1.3',
             JIMBase.USER: {JIMBase.ACCOUNT_NAME: 'Guest'}}), self.err_dict)

    def test_wrong_action(self):
        """Ошибка если ACTION неизвестен"""
        self.assertEqual(JIMServer.process_client_message(
            {JIMBase.ACTION: 'Wrong',
             JIMBase.TIME: '1.3',
             JIMBase.USER: {JIMBase.ACCOUNT_NAME: 'Guest'}}), self.err_dict)

    def test_no_time(self):
        """Ошибка, если  запрос не содержит TIME"""
        self.assertEqual(JIMServer.process_client_message(
            {JIMBase.ACTION: JIMBase.PRESENCE,
             JIMBase.USER: {JIMBase.ACCOUNT_NAME: 'Guest'}}), self.err_dict)

    def test_no_user(self):
        """Ошибка - нет ACCOUNT_NAME"""
        self.assertEqual(JIMServer.process_client_message(
            {JIMBase.ACTION: JIMBase.PRESENCE,
             JIMBase.TIME: '1.1'}), self.err_dict)

    def test_unknown_user(self):
        """Ошибка - не Guest"""
        self.assertEqual(JIMServer.process_client_message(
            {JIMBase.ACTION: JIMBase.PRESENCE,
             JIMBase.TIME: 1.1,
             JIMBase.USER: {JIMBase.ACCOUNT_NAME: 'Guest1'}}), self.err_dict)

    def test_ok_dict(self):
        """Корректный запрос"""
        self.assertEqual(JIMServer.process_client_message(
            {JIMBase.ACTION: JIMBase.PRESENCE,
             JIMBase.TIME: 1.1,
             JIMBase.USER: {JIMBase.ACCOUNT_NAME: 'Guest'}}), self.ok_dict)


if __name__ == '__main__':
    unittest.main()



from dis import get_instructions


class ServerInspector(type):
    """
    Метакласс для проверки соответствия сервера.
    """
    def __init__(cls, clssname, parents, attributes):
        # clssname - экземпляр метакласса - JIMServer
        # parents - кортеж базовых классов - ()
        # attributes - словарь атрибутов и методов экземпляра метакласса

        # Список методов, используемых в функциях класса:
        methods = []
        # Атрибуты, которые используются в функциях классов
        attrs = []
        # перебираем атрибуты
        for func in attributes:
            try:
                # смотрим что это - функция или нет
                ret_attr = get_instructions(attributes[func])
                # Если не функция то ловим исключение
            except TypeError:
                pass
            else:
                # Раз функция разбираем код, получая ее методы и атрибуты.
                # Имя разбираемой функции
                print(func)
                for i, instr in enumerate(ret_attr):
                    # i - порядковый номер (чтобы не запутаться)
                    # instr - Instruction(opname='LOAD_ATTR', opcode=106, arg=2, argval='listen_port',
                    # argrepr='listen_port', offset=8, starts_line=None, is_jump_target=False)
                    # opername - имя для операции
                    print(i, instr)
                    if instr.opname == 'LOAD_GLOBAL':
                        if instr.argval not in methods:
                            # заполняем список методами, использующимися в функциях класса
                            methods.append(instr.argval)
                    elif instr.opname == 'LOAD_ATTR':
                        if instr.argval not in attrs:
                            # заполняем список атрибутами, использующимися в функциях класса
                            attrs.append(instr.argval)
        print(methods)
        # Если обнаружено использование недопустимого метода connect, бросаем исключение:
        if 'connect' in methods:
            raise TypeError('Использование метода connect недопустимо в серверном классе')
        # Если сокет не инициализировался константами SOCK_STREAM(TCP) AF_INET(IPv4), тоже исключение.
        if not ('SOCK_STREAM' in attrs and 'AF_INET' in attrs):
            raise TypeError('Проверьте сокет - он будто не инициализирован.')
        # Обязательно вызываем конструктор родителя:
        super().__init__(clssname, parents, attributes)


class ClientInspector(type):
    """
    Метакласс для проверки корректности клиентов.
    """
    def __init__(cls, clssname, parents, attributes):
        # Список методов, которые используются в функциях класса:
        methods = []
        for func in attributes:
            try:
                ret_attr = get_instructions(attributes[func])
                # Если не функция то ловим исключение
            except TypeError:
                pass
            else:
                # Раз функция разбираем код, получая используемые методы.
                for i in ret_attr:
                    if i.opname == 'LOAD_GLOBAL':
                        if i.argval not in methods:
                            methods.append(i.argval)
        # Если обнаружено использование недопустимого метода accept, listen (socket - не запрещен) бросаем исключение:
        for command in ('accept', 'listen'):
            if command in methods:
                raise TypeError('В классе обнаружены запрещенные методы!')
        # Вызов get_message или send_message из utils считаем корректным использованием сокетов
        if 'get_message' in methods or 'send_message' in methods:
            raise TypeError('Функции должны работать с JSON_Messenger!')
        else:
            pass
        super().__init__(clssname, parents, attributes)
